@@ -1,31 +1,7 @@
 /*
- * [The "BSD license"]
- *  Copyright (c) 2012 Terence Parr
- *  Copyright (c) 2012 Sam Harwell
- *  All rights reserved.
- *
- *  Redistribution and use in source and binary forms, with or without
- *  modification, are permitted provided that the following conditions
- *  are met:
- *
- *  1. Redistributions of source code must retain the above copyright
- *     notice, this list of conditions and the following disclaimer.
- *  2. Redistributions in binary form must reproduce the above copyright
- *     notice, this list of conditions and the following disclaimer in the
- *     documentation and/or other materials provided with the distribution.
- *  3. The name of the author may not be used to endorse or promote products
- *     derived from this software without specific prior written permission.
- *
- *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- *  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+ * Use of this file is governed by the BSD 3-clause license that
+ * can be found in the LICENSE.txt file in the project root.
  */
 package org.antlr.v4.runtime;
 
@@ -120,10 +96,10 @@ public class TokenStreamRewriter {
 	// Define the rewrite operation hierarchy
 
 	public class RewriteOperation {
-	/** What index into rewrites List are we? */
-	protected int instructionIndex;
-	/** Token buffer index. */
-	protected int index;
+		/** What index into rewrites List are we? */
+		protected int instructionIndex;
+		/** Token buffer index. */
+		protected int index;
 		protected Object text;
 
 		protected RewriteOperation(int index) {
@@ -166,6 +142,16 @@ public class TokenStreamRewriter {
 		}
 	}
 
+	/** Distinguish between insert after/before to do the "insert afters"
+	 *  first and then the "insert befores" at same index. Implementation
+	 *  of "insert after" is "insert before index+1".
+	 */
+    class InsertAfterOp extends InsertBeforeOp {
+        public InsertAfterOp(int index, Object text) {
+            super(index+1, text); // insert after is insert before index+1
+        }
+    }
+
 	/** I'm going to try replacing range from x..y with (y-x)+1 ReplaceOp
 	 *  instructions.
 	 */
@@ -255,7 +241,10 @@ public class TokenStreamRewriter {
 
 	public void insertAfter(String programName, int index, Object text) {
 		// to insert after, just insert before next index (even if past end)
-		insertBefore(programName,index+1, text);
+        RewriteOperation op = new InsertAfterOp(index, text);
+        List<RewriteOperation> rewrites = getProgram(programName);
+        op.instructionIndex = rewrites.size();
+        rewrites.add(op);
 	}
 
 	public void insertBefore(Token t, Object text) {
@@ -520,8 +509,6 @@ public class TokenStreamRewriter {
 				// throw exception unless disjoint or identical
 				boolean disjoint =
 					prevRop.lastIndex<rop.index || prevRop.index > rop.lastIndex;
-				boolean same =
-					prevRop.index==rop.index && prevRop.lastIndex==rop.lastIndex;
 				// Delete special case of replace (text==null):
 				// D.i-j.u D.x-y.v	| boundaries overlap	combine to max(min)..max(right)
 				if ( prevRop.text==null && rop.text==null && !disjoint ) {
@@ -531,7 +518,7 @@ public class TokenStreamRewriter {
 					rop.lastIndex = Math.max(prevRop.lastIndex, rop.lastIndex);
 					System.out.println("new rop "+rop);
 				}
-				else if ( !disjoint && !same ) {
+				else if ( !disjoint ) {
 					throw new IllegalArgumentException("replace op boundaries of "+rop+" overlap with previous "+prevRop);
 				}
 			}
@@ -546,12 +533,18 @@ public class TokenStreamRewriter {
 			// combine current insert with prior if any at same index
 			List<? extends InsertBeforeOp> prevInserts = getKindOfOps(rewrites, InsertBeforeOp.class, i);
 			for (InsertBeforeOp prevIop : prevInserts) {
-				if ( prevIop.index == iop.index ) { // combine objects
-					// convert to strings...we're in process of toString'ing
-					// whole token buffer so no lazy eval issue with any templates
-					iop.text = catOpText(iop.text,prevIop.text);
-					// delete redundant prior insert
-					rewrites.set(prevIop.instructionIndex, null);
+				if ( prevIop.index==iop.index ) {
+					if ( InsertAfterOp.class.isInstance(prevIop) ) {
+						iop.text = catOpText(prevIop.text, iop.text);
+						rewrites.set(prevIop.instructionIndex, null);
+					}
+					else if ( InsertBeforeOp.class.isInstance(prevIop) ) { // combine objects
+						// convert to strings...we're in process of toString'ing
+						// whole token buffer so no lazy eval issue with any templates
+						iop.text = catOpText(iop.text, prevIop.text);
+						// delete redundant prior insert
+						rewrites.set(prevIop.instructionIndex, null);
+					}
 				}
 			}
 			// look for replaces where iop.index is in range; error
