@@ -1,31 +1,7 @@
 /*
- * [The "BSD license"]
- *  Copyright (c) 2012 Terence Parr
- *  Copyright (c) 2012 Sam Harwell
- *  All rights reserved.
- *
- *  Redistribution and use in source and binary forms, with or without
- *  modification, are permitted provided that the following conditions
- *  are met:
- *
- *  1. Redistributions of source code must retain the above copyright
- *     notice, this list of conditions and the following disclaimer.
- *  2. Redistributions in binary form must reproduce the above copyright
- *     notice, this list of conditions and the following disclaimer in the
- *     documentation and/or other materials provided with the distribution.
- *  3. The name of the author may not be used to endorse or promote products
- *     derived from this software without specific prior written permission.
- *
- *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- *  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+ * Use of this file is governed by the BSD 3-clause license that
+ * can be found in the LICENSE.txt file in the project root.
  */
 
 package org.antlr.v4.runtime.misc;
@@ -177,4 +153,58 @@ public class Utils {
 		}
 		return s;
 	}
+
+	/** @since 4.6 */
+	public static String expandTabs(String s, int tabSize) {
+		if ( s==null ) return null;
+		StringBuilder buf = new StringBuilder();
+		int col = 0;
+		for (int i = 0; i<s.length(); i++) {
+			char c = s.charAt(i);
+			switch ( c ) {
+				case '\n' :
+					col = 0;
+					buf.append(c);
+					break;
+				case '\t' :
+					int n = tabSize-col%tabSize;
+					col+=n;
+					buf.append(spaces(n));
+					break;
+				default :
+					col++;
+					buf.append(c);
+					break;
+			}
+		}
+		return buf.toString();
+	}
+
+	/** @since 4.6 */
+	public static String spaces(int n) {
+		return sequence(n, " ");
+	}
+
+	/** @since 4.6 */
+	public static String newlines(int n) {
+		return sequence(n, "\n");
+	}
+
+	/** @since 4.6 */
+	public static String sequence(int n, String s) {
+		StringBuilder buf = new StringBuilder();
+		for (int sp=1; sp<=n; sp++) buf.append(s);
+		return buf.toString();
+	}
+
+	/** @since 4.6 */
+	public static int count(String s, char x) {
+		int n = 0;
+		for (int i = 0; i<s.length(); i++) {
+			if ( s.charAt(i)==x ) {
+				n++;
+			}
+		}
+		return n;
+	}
 }
