@@ -1,31 +1,7 @@
 /*
- * [The "BSD license"]
- *  Copyright (c) 2012 Terence Parr
- *  Copyright (c) 2012 Sam Harwell
- *  All rights reserved.
- *
- *  Redistribution and use in source and binary forms, with or without
- *  modification, are permitted provided that the following conditions
- *  are met:
- *
- *  1. Redistributions of source code must retain the above copyright
- *     notice, this list of conditions and the following disclaimer.
- *  2. Redistributions in binary form must reproduce the above copyright
- *     notice, this list of conditions and the following disclaimer in the
- *     documentation and/or other materials provided with the distribution.
- *  3. The name of the author may not be used to endorse or promote products
- *     derived from this software without specific prior written permission.
- *
- *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- *  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+ * Use of this file is governed by the BSD 3-clause license that
+ * can be found in the LICENSE.txt file in the project root.
  */
 package org.antlr.v4.test.tool;
 
@@ -36,14 +12,14 @@ import org.antlr.v4.automata.ParserATNFactory;
 import org.antlr.v4.parse.ANTLRParser;
 import org.antlr.v4.runtime.atn.ATN;
 import org.antlr.v4.runtime.atn.ATNState;
-import org.antlr.v4.test.runtime.java.BaseTest;
-import org.antlr.v4.test.runtime.java.ErrorQueue;
+import org.antlr.v4.test.runtime.ErrorQueue;
 import org.antlr.v4.tool.ErrorType;
 import org.antlr.v4.tool.Grammar;
 import org.antlr.v4.tool.LexerGrammar;
 import org.antlr.v4.tool.ast.GrammarAST;
 import org.antlr.v4.tool.ast.GrammarRootAST;
 import org.antlr.v4.tool.ast.RuleAST;
+import org.junit.Before;
 import org.junit.Test;
 
 import java.util.Arrays;
@@ -54,7 +30,13 @@ import java.util.Map;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 
-public class TestATNConstruction extends BaseTest {
+public class TestATNConstruction extends BaseJavaToolTest {
+	@Before
+	@Override
+	public void testSetUp() throws Exception {
+		super.testSetUp();
+	}
+
 	@Test
 	public void testA() throws Exception {
 		Grammar g = new Grammar(
@@ -358,6 +340,20 @@ public class TestATNConstruction extends BaseTest {
 				"RuleStop_a_1-EOF->s9\n";
 		checkRuleATN(g, "a", expecting);
 	}
+	@Test public void testEmptyOrEmpty() throws Exception {
+		Grammar g = new Grammar(
+			"parser grammar P;\n"+
+			"a : | ;");
+		String expecting =
+			"RuleStart_a_0->BlockStart_4\n"+
+			"BlockStart_4->s2\n"+
+			"BlockStart_4->s3\n"+
+			"s2->BlockEnd_5\n"+
+			"s3->BlockEnd_5\n"+
+			"BlockEnd_5->RuleStop_a_1\n"+
+			"RuleStop_a_1-EOF->s6\n";
+		checkRuleATN(g, "a", expecting);
+	}
 	@Test public void testAStar() throws Exception {
 		Grammar g = new Grammar(
 			"parser grammar P;\n"+
@@ -464,6 +460,65 @@ public class TestATNConstruction extends BaseTest {
 		assertTrue(!threwException);
 	}
 
+	/** Test for https://github.com/antlr/antlr4/issues/1369
+	 *  Repeated edges:
+
+	 RuleStop_e_3->BlockEnd_26
+	 RuleStop_e_3->BlockEnd_26
+	 RuleStop_e_3->BlockEnd_26
+
+	 * @throws Exception
+	 */
+	@Test public void testForRepeatedTransitionsToStopState() throws Exception {
+		String gstr =
+			"grammar T;\n"+
+			"\t s : e EOF;\n"+
+			"\t e :<assoc=right> e '*' e\n"+
+			"\t   |<assoc=right> e '+' e\n"+
+			"\t   |<assoc=right> e '?' e ':' e\n"+
+			"\t   |<assoc=right> e '=' e\n"+
+			"\t   | ID\n"+
+			"\t   ;\n"+
+			"\t ID : 'a'..'z'+ ;\n"+
+			"\t WS : (' '|'\\n') -> skip ;";
+		Grammar g = new Grammar(gstr);
+		String expecting =
+			"RuleStart_e_2->s7\n"+
+			"s7-action_1:-1->s8\n"+
+			"s8-ID->s9\n"+
+			"s9->StarLoopEntry_27\n"+
+			"StarLoopEntry_27->StarBlockStart_25\n"+
+			"StarLoopEntry_27->s28\n"+
+			"StarBlockStart_25->s10\n"+
+			"StarBlockStart_25->s13\n"+
+			"StarBlockStart_25->s16\n"+
+			"StarBlockStart_25->s22\n"+
+			"s28->RuleStop_e_3\n"+
+			"s10-5 >= _p->s11\n"+
+			"s13-4 >= _p->s14\n"+
+			"s16-3 >= _p->s17\n"+
+			"s22-2 >= _p->s23\n"+
+			"RuleStop_e_3->s5\n"+
+			"RuleStop_e_3->BlockEnd_26\n"+
+			"RuleStop_e_3->s19\n"+
+			"RuleStop_e_3->s21\n"+
+			"s11-'*'->s12\n"+
+			"s14-'+'->s15\n"+
+			"s17-'?'->s18\n"+
+			"s23-'='->s24\n"+
+			"s12-e->RuleStart_e_2\n"+
+			"s15-e->RuleStart_e_2\n"+
+			"s18-e->RuleStart_e_2\n"+
+			"s24-e->RuleStart_e_2\n"+
+			"BlockEnd_26->StarLoopBack_29\n"+
+			"s19-':'->s20\n"+
+			"StarLoopBack_29->StarLoopEntry_27\n"+
+			"s20-e->RuleStart_e_2\n"+
+			"s21->BlockEnd_26\n";
+		checkRuleATN(g, "e", expecting);
+	}
+
+
 /*
 	@Test public void testMultiplePredicates() throws Exception {
 		Grammar g = new Grammar(
@@ -610,311 +665,8 @@ public class TestATNConstruction extends BaseTest {
 			"RuleStop_a_1-EOF->s4\n";
 		checkRule(g, "a", expecting);
 	}
-	// AUTO BACKTRACKING STUFF
-	@Test public void testAutoBacktracking_RuleBlock() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : 'a'{;}|'b';"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s1->.s9\n" +
-			".s10-'b'->.s11\n" +
-			".s11->.s6\n" +
-			".s2-{synpred1_t}?->.s3\n" +
-			".s3-'a'->.s4\n" +
-			".s4-{}->.s5\n" +
-			".s5->.s6\n" +
-			".s6->:s7\n" +
-			".s9->.s10\n" +
-			":s7-EOF->.s8\n";
-		checkRule(g, "a", expecting);
-	}
-	@Test public void testAutoBacktracking_RuleSetBlock() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : 'a'|'b';"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s2-'a'..'b'->.s3\n" +
-			".s3->:s4\n" +
-			":s4-EOF->.s5\n";
-		checkRule(g, "a", expecting);
-	}
-	@Test public void testAutoBacktracking_SimpleBlock() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : ('a'{;}|'b') ;"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s10->.s11\n" +
-			".s11-'b'->.s12\n" +
-			".s12->.s7\n" +
-			".s2->.s10\n" +
-			".s2->.s3\n" +
-			".s3-{synpred1_t}?->.s4\n" +
-			".s4-'a'->.s5\n" +
-			".s5-{}->.s6\n" +
-			".s6->.s7\n" +
-			".s7->:s8\n" +
-			":s8-EOF->.s9\n";
-		checkRule(g, "a", expecting);
-	}
-	@Test public void testAutoBacktracking_SetBlock() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : ('a'|'b') ;"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s2-'a'..'b'->.s3\n" +
-			".s3->:s4\n" +
-			":s4-EOF->.s5\n";
-		checkRule(g, "a", expecting);
-	}
-	@Test public void testAutoBacktracking_StarBlock() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : ('a'{;}|'b')* ;"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s12->.s13\n" +
-			".s13-{synpred2_t}?->.s14\n" +
-			".s14-'b'->.s15\n" +
-			".s15->.s8\n" +
-			".s16->.s9\n" +
-			".s2->.s16\n" +
-			".s2->.s3\n" +
-			".s3->.s12\n" +
-			".s3->.s4\n" +
-			".s4-{synpred1_t}?->.s5\n" +
-			".s5-'a'->.s6\n" +
-			".s6-{}->.s7\n" +
-			".s7->.s8\n" +
-			".s8->.s3\n" +
-			".s8->.s9\n" +
-			".s9->:s10\n" +
-			":s10-EOF->.s11\n";
-		checkRule(g, "a", expecting);
-	}
-	@Test public void testAutoBacktracking_StarSetBlock_IgnoresPreds() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : ('a'|'b')* ;"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s2->.s3\n" +
-			".s2->.s9\n" +
-			".s3->.s4\n" +
-			".s4-'a'..'b'->.s5\n" +
-			".s5->.s3\n" +
-			".s5->.s6\n" +
-			".s6->:s7\n" +
-			".s9->.s6\n" +
-			":s7-EOF->.s8\n";
-		checkRule(g, "a", expecting);
-	}
-	@Test public void testAutoBacktracking_StarSetBlock() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : ('a'|'b'{;})* ;"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s11->.s12\n" +
-			".s12-{synpred2_t}?->.s13\n" +
-			".s13-'b'->.s14\n" +
-			".s14-{}->.s15\n" +
-			".s15->.s7\n" +
-			".s16->.s8\n" +
-			".s2->.s16\n" +
-			".s2->.s3\n" +
-			".s3->.s11\n" +
-			".s3->.s4\n" +
-			".s4-{synpred1_t}?->.s5\n" +
-			".s5-'a'->.s6\n" +
-			".s6->.s7\n" +
-			".s7->.s3\n" +
-			".s7->.s8\n" +
-			".s8->:s9\n" +
-			":s9-EOF->.s10\n";
-		checkRule(g, "a", expecting);
-	}
-	@Test public void testAutoBacktracking_StarBlock1Alt() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : ('a')* ;"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s10->.s7\n" +
-			".s2->.s10\n" +
-			".s2->.s3\n" +
-			".s3->.s4\n" +
-			".s4-{synpred1_t}?->.s5\n" +
-			".s5-'a'->.s6\n" +
-			".s6->.s3\n" +
-			".s6->.s7\n" +
-			".s7->:s8\n" +
-			":s8-EOF->.s9\n";
-		checkRule(g, "a", expecting);
-	}
-	@Test public void testAutoBacktracking_PlusBlock() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : ('a'{;}|'b')+ ;"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s12->.s13\n" +
-			".s13-{synpred2_t}?->.s14\n" +
-			".s14-'b'->.s15\n" +
-			".s15->.s8\n" +
-			".s2->.s3\n" +
-			".s3->.s12\n" +
-			".s3->.s4\n" +
-			".s4-{synpred1_t}?->.s5\n" +
-			".s5-'a'->.s6\n" +
-			".s6-{}->.s7\n" +
-			".s7->.s8\n" +
-			".s8->.s3\n" +
-			".s8->.s9\n" +
-			".s9->:s10\n" +
-			":s10-EOF->.s11\n";
-		checkRule(g, "a", expecting);
-	}
-	@Test public void testAutoBacktracking_PlusSetBlock() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : ('a'|'b'{;})+ ;"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s11->.s12\n" +
-			".s12-{synpred2_t}?->.s13\n" +
-			".s13-'b'->.s14\n" +
-			".s14-{}->.s15\n" +
-			".s15->.s7\n" +
-			".s2->.s3\n" +
-			".s3->.s11\n" +
-			".s3->.s4\n" +
-			".s4-{synpred1_t}?->.s5\n" +
-			".s5-'a'->.s6\n" +
-			".s6->.s7\n" +
-			".s7->.s3\n" +
-			".s7->.s8\n" +
-			".s8->:s9\n" +
-			":s9-EOF->.s10\n";
-		checkRule(g, "a", expecting);
-	}
-	@Test public void testAutoBacktracking_PlusBlock1Alt() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : ('a')+ ;"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s2->.s3\n" +
-			".s3->.s4\n" +
-			".s4-{synpred1_t}?->.s5\n" +
-			".s5-'a'->.s6\n" +
-			".s6->.s3\n" +
-			".s6->.s7\n" +
-			".s7->:s8\n" +
-			":s8-EOF->.s9\n";
-		checkRule(g, "a", expecting);
-	}
-	@Test public void testAutoBacktracking_OptionalBlock2Alts() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : ('a'{;}|'b')?;"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s10->.s11\n" +
-			".s10->.s14\n" +
-			".s11-{synpred2_t}?->.s12\n" +
-			".s12-'b'->.s13\n" +
-			".s13->.s7\n" +
-			".s14->.s7\n" +
-			".s2->.s10\n" +
-			".s2->.s3\n" +
-			".s3-{synpred1_t}?->.s4\n" +
-			".s4-'a'->.s5\n" +
-			".s5-{}->.s6\n" +
-			".s6->.s7\n" +
-			".s7->:s8\n" +
-			":s8-EOF->.s9\n";
-		checkRule(g, "a", expecting);
-	}
-	@Test public void testAutoBacktracking_OptionalBlock1Alt() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : ('a')?;"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s2->.s3\n" +
-			".s2->.s9\n" +
-			".s3-{synpred1_t}?->.s4\n" +
-			".s4-'a'->.s5\n" +
-			".s5->.s6\n" +
-			".s6->:s7\n" +
-			".s9->.s6\n" +
-			":s7-EOF->.s8\n";
-		checkRule(g, "a", expecting);
-	}
-	@Test public void testAutoBacktracking_ExistingPred() throws Exception {
-		Grammar g = new Grammar(
-			"grammar t;\n" +
-			"options {backtrack=true;}\n"+
-			"a : ('a')=> 'a' | 'b';"
-		);
-		String expecting =
-			".s0->.s1\n" +
-			".s1->.s2\n" +
-			".s1->.s8\n" +
-			".s10->.s5\n" +
-			".s2-{synpred1_t}?->.s3\n" +
-			".s3-'a'->.s4\n" +
-			".s4->.s5\n" +
-			".s5->:s6\n" +
-			".s8->.s9\n" +
-			".s9-'b'->.s10\n" +
-			":s6-EOF->.s7\n";
-		checkRule(g, "a", expecting);
-	}
-*/
+	*/
+
 	@Test public void testDefaultMode() throws Exception {
 		LexerGrammar g = new LexerGrammar(
 			"lexer grammar L;\n"+
@@ -934,6 +686,7 @@ public class TestATNConstruction extends BaseTest {
 				"s13->RuleStop_X_5\n";
 		checkTokensRule(g, "DEFAULT_MODE", expecting);
 	}
+
 	@Test public void testMode() throws Exception {
 		LexerGrammar g = new LexerGrammar(
 			"lexer grammar L;\n"+
