@@ -1,12 +1,23 @@
+/*
+ * Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+ * Use of this file is governed by the BSD 3-clause license that
+ * can be found in the LICENSE.txt file in the project root.
+ */
+
 package org.antlr.v4.test.tool;
 
-import org.antlr.v4.test.runtime.java.BaseTest;
+import org.junit.Before;
 import org.junit.Test;
 
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 
-public class TestDollarParser extends BaseTest {
+public class TestDollarParser extends BaseJavaToolTest {
+	@Before
+	@Override
+	public void testSetUp() throws Exception {
+		super.testSetUp();
+	}
 
 	@Test
 	public void testSimpleCall() throws Exception {
@@ -14,7 +25,8 @@ public class TestDollarParser extends BaseTest {
 	                  "a : ID  { System.out.println( $parser.getSourceName() ); }\n" +
 	                  "  ;\n" +
 	                  "ID : 'a'..'z'+ ;\n";
-		String found = execParser("T.g4", grammar, "TParser", "TLexer", "a", "x", true);
+		String found = execParser("T.g4", grammar, "TParser", "TLexer",
+		                          null, null, "a", "x", true);
 		assertTrue(found.indexOf(this.getClass().getSimpleName())>=0);
 		assertNull(this.stderrDuringParse);
 	}
