@@ -1,31 +1,7 @@
 /*
- * [The "BSD license"]
- *  Copyright (c) 2012 Terence Parr
- *  Copyright (c) 2012 Sam Harwell
- *  All rights reserved.
- *
- *  Redistribution and use in source and binary forms, with or without
- *  modification, are permitted provided that the following conditions
- *  are met:
- *
- *  1. Redistributions of source code must retain the above copyright
- *     notice, this list of conditions and the following disclaimer.
- *  2. Redistributions in binary form must reproduce the above copyright
- *     notice, this list of conditions and the following disclaimer in the
- *     documentation and/or other materials provided with the distribution.
- *  3. The name of the author may not be used to endorse or promote products
- *     derived from this software without specific prior written permission.
- *
- *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- *  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- *  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+ * Use of this file is governed by the BSD 3-clause license that
+ * can be found in the LICENSE.txt file in the project root.
  */
 
 package org.antlr.v4.codegen;
@@ -211,26 +187,21 @@ public class ActionTranslator implements ActionSplitterListener {
 			return;
 		}
 		Attribute a = node.resolver.resolveToAttribute(x.getText(), y.getText(), node);
+		if ( a==null ) {
+			// Added in response to https://github.com/antlr/antlr4/issues/1211
+			gen.g.tool.errMgr.grammarError(ErrorType.UNKNOWN_SIMPLE_ATTRIBUTE,
+			                               gen.g.fileName, x,
+			                               x.getText(),
+			                               "rule");
+			return;
+		}
 		switch ( a.dict.type ) {
 			case ARG: chunks.add(new ArgRef(nodeContext,y.getText())); break; // has to be current rule
 			case RET:
-				if ( factory.getCurrentRuleFunction()!=null &&
-					factory.getCurrentRuleFunction().name.equals(x.getText()) )
-				{
-					chunks.add(new RetValueRef(rf.ruleCtx, y.getText())); break;
-				}
-				else {
-					chunks.add(new QRetValueRef(nodeContext, getRuleLabel(x.getText()), y.getText())); break;
-				}
+				chunks.add(new QRetValueRef(nodeContext, getRuleLabel(x.getText()), y.getText()));
+				break;
 			case PREDEFINED_RULE:
-				if ( factory.getCurrentRuleFunction()!=null &&
-					factory.getCurrentRuleFunction().name.equals(x.getText()) )
-				{
-					chunks.add(getRulePropertyRef(y));
-				}
-				else {
-					chunks.add(getRulePropertyRef(x, y));
-				}
+				chunks.add(getRulePropertyRef(x, y));
 				break;
 			case TOKEN:
 				chunks.add(getTokenPropertyRef(x, y));
